import { VerificationError, apply_number, apply_string, apply_boolean, apply_void } from "./ts_base";
import TestAtom, { apply_TestAtom } from "./TestAtom";
import TestEnum, { apply_TestEnum } from "./TestEnum";
export default class Test {
   atom?: TestAtom;
   array?: TestAtom[];
   enumValue?: TestEnum;
   map?: { [key: number]: TestAtom };

   constructor(init?:Partial<Test>){
      if(init){
         if(init["atom"])
            this.atom = init["atom"];
         if(init["array"])
            this.array = init["array"];
         if(init["enumValue"])
            this.enumValue = init["enumValue"];
         if(init["map"])
            this.map = init["map"];
      }
   }


   static apply(data: Test) {
      apply_Test(data);
   }
}

export function apply_Test(data: Test): Test {
   if(typeof data !== "object") throw new VerificationError("Test", undefined, data);
   let res = new Test() as any;
   if(data["atom"] !== null && data["atom"] !== undefined) {
      res["atom"] = apply_TestAtom(data["atom"])
   }
   if(data["array"] !== null && data["array"] !== undefined) {
      if(!Array.isArray(data["array"])) throw new VerificationError("array", "array", data["array"]);
      res["array"] = data["array"].map(elm=>
         apply_TestAtom(elm)
      )
   }
   if(data["enumValue"] !== null && data["enumValue"] !== undefined) {
      res["enumValue"] = apply_TestEnum(data["enumValue"])
   }
   if(data["map"] !== null && data["map"] !== undefined) {
      if(typeof data["map"] !== "object") throw new VerificationError("map", "map", data["map"]);
      res["map"] = {}
      Object.entries(data["map"]).forEach(([key, val]) => res["map"][key] = apply_TestAtom(val))
   }
   return res;
}
